package com.nisum.us;

import com.nisum.us.dto.OrderDTO;
import io.github.resilience4j.bulkhead.annotation.Bulkhead;
import io.github.resilience4j.circuitbreaker.annotation.CircuitBreaker;
import io.github.resilience4j.ratelimiter.annotation.RateLimiter;
import io.github.resilience4j.retry.annotation.Retry;
import io.github.resilience4j.timelimiter.annotation.TimeLimiter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.annotation.Caching;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Lazy;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

@SpringBootApplication
@RestController
@RequestMapping("/user-service")
public class UserServiceApplication {

    @Autowired
    @Lazy
    private RestTemplate restTemplate;

    public static final String USER_SERVICE = "userService";

    private static final String BASEURL = "http://localhost:9393/orders";

    private int attempt = 1;

    @GetMapping
    public List<OrderDTO> testMethod() {
        return restTemplate.getForObject(BASEURL, ArrayList.class);
    }

    @GetMapping("/displayOrders")
    @CircuitBreaker(name = USER_SERVICE, fallbackMethod = "getAllAvailableProducts")
    public List<OrderDTO> displayOrders(@RequestParam("category") String category) {
        String url = category == null ? BASEURL : BASEURL + "/" + category;
        return restTemplate.getForObject(url, ArrayList.class);
    }

    @GetMapping("/retryDisplayOrders")
    @Retry(name = USER_SERVICE, fallbackMethod = "getAllAvailableProducts")
    public List<OrderDTO> displayOrders2(@RequestParam("category") String category) {
        String url = category == null ? BASEURL : BASEURL + "/" + category;
        System.out.println("retry method called " + attempt++ + " times " + " at " + new Date());
        return restTemplate.getForObject(url, ArrayList.class);
    }

    @GetMapping("/bulkHeadDisplayOrders")
    @Bulkhead(name = USER_SERVICE, fallbackMethod = "getAllAvailableProducts")
    public List<OrderDTO> displayOrders3(@RequestParam("category") String category) {
        String url = category == null ? BASEURL : BASEURL + "/" + category;
        System.out.println("bulkhead method called at " + new Date());
        try {
            Thread.sleep(4000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return restTemplate.getForObject(url, ArrayList.class);
    }

    @GetMapping("/rateLimiterDisplayOrders")
    @RateLimiter(name = USER_SERVICE, fallbackMethod = "getAllAvailableProducts")
    public List<OrderDTO> displayOrders4(@RequestParam("category") String category) {
        String url = category == null ? BASEURL : BASEURL + "/" + category;
        return restTemplate.getForObject(url, ArrayList.class);
    }

    @GetMapping("/timeLimitDisplayOrders")
    @TimeLimiter(name = USER_SERVICE)
    public List<OrderDTO> displayOrders5(@RequestParam("category") String category) {
        String url = category == null ? BASEURL : BASEURL + "/" + category;
        System.out.println("bulkhead method called at " + new Date());
        try {
            Thread.sleep(4000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return restTemplate.getForObject(url, ArrayList.class);
    }

    @GetMapping("/aggregate")
    public List<OrderDTO> displayOrders6(@RequestParam("category") String category) {
        String url = category == null ? BASEURL : BASEURL + "/electronics"  ;
        ArrayList<OrderDTO> forObject1 = restTemplate.getForObject(url, ArrayList.class);
        url = category == null ? BASEURL : BASEURL + "/clothes";
        ArrayList<OrderDTO> forObject = restTemplate.getForObject(url, ArrayList.class);
        List<OrderDTO> aggList = new ArrayList<>();
        aggList.addAll(forObject);
        aggList.addAll(forObject1);
        return aggList;
    }

    public List<OrderDTO> getAllAvailableProducts(Exception e) {
        return Stream.of(
                new OrderDTO(119, "dummy11", "electronics", "white", 45000),
                new OrderDTO(345, "dummy12", "electronics", "black", 7000),
                new OrderDTO(475, "dummy13", "electronics", "black", 13000),
                new OrderDTO(574, "dummy14", "foot wear", "black & white", 4600),
                new OrderDTO(678, "dummy15", "kitchen", "blue", 999),
                new OrderDTO(532, "dummy16", "kitchen", "gray", 745)
        ).collect(Collectors.toList());
    }

    @GetMapping("/{id}")
    @Cacheable(key = "#id",value = "OrderDTO",unless = "#result.price > 1000")
    public OrderDTO findProduct(@PathVariable int id) {
        return null;
    }

    @DeleteMapping("/{id}")
    @CacheEvict(key = "#id",value = "Product")
    public String remove(@PathVariable int id) {
        return null;
    }

    public static void main(String[] args) {

        SpringApplication.run(UserServiceApplication.class, args);

        //bulkhead demo
//        int i = 1;
//        IntStream.range(i, 10).parallel().forEach(t -> {
//            new RestTemplate().getForObject("http://localhost:9191/user-service/bulkHeadDisplayOrders?category=", String.class);
//        });
    }

    @Bean
    public RestTemplate restTemplate() {
        return new RestTemplate();
    }
}
